package cn.com.ty.lift.business.common;

import cn.com.ty.lift.business.framework.conf.SystemConfiguration;
import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.library.dao.entity.LiftBrand;
import cn.com.ty.lift.common.constants.RedisConstants;
import cn.com.ty.lift.common.model.AreaCode;
import cn.com.ty.lift.common.utils.AliyunOSSUtil;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author bieao
 * @date 2019/12/19
 * @description 公共初始化数据
 */
@Slf4j
@AllArgsConstructor
@RestController
@RequestMapping("common")
public class CommonController {

    @Resource
    private RedisTemplate<String, Object> redisTemplate;

    private AliyunOSSUtil aliyunOSSUtil;

    private SystemConfiguration systemConfiguration;

    //上传文件最大5M
    private final long uploadMaxSize = 5 * 1024 * 1024;

    /**
     * @return RestResponse 省、市、区树形结构列表
     * @description 查询省、市、区树形结构列表
     * @date 2019/12/4 10:23 AM
     */
    @PostMapping("area/list")
    @SuppressWarnings("unchecked")
    public RestResponse areaList() {
        List<AreaCode> areaCodeList = (List<AreaCode>)redisTemplate.opsForValue().get(RedisConstants.RK_AREA_LIST);
        if (ObjectUtil.isEmpty(areaCodeList)) {
            return RestResponse.success();
        }
        return RestResponse.success(areaCodeList, MessageUtils.get("msg.query.success"));
    }

    /**
     * @return RestResponse 电梯品牌列表
     * @description 查询电梯品牌列表
     * @date 2019/12/10 10:23 AM
     */
    @PostMapping("brand/list")
    @SuppressWarnings(value = "unchecked")
    public RestResponse list(){
        List<LiftBrand> liftBrandList = (List<LiftBrand>) redisTemplate.opsForValue().get(RedisConstants.RK_LIFT_BRAND_LIST);
        if (ObjectUtil.isEmpty(liftBrandList)) {
            return RestResponse.success();
        }
        return RestResponse.success(liftBrandList, MessageUtils.get("msg.query.success"));
    }

    /**
     * 上传图片到aliyun OSS，返回图片的地址
     * @param file
     * @return
     * @throws Exception
     */
    @PostMapping("upload")
    public RestResponse upload(@RequestParam("file") MultipartFile file) throws Exception{
        if(null == file || file.isEmpty()){
            return RestResponse.fail("没有解析到文件");
        }
        if(file.getSize() > uploadMaxSize){
            return RestResponse.fail("文件大小不超过5M");
        }
        // 获取文件名，带后缀
        String originalFilename = file.getOriginalFilename();
        log.info("上传文件，原文件名：" + originalFilename);
        // 获取文件的后缀格式
        String fileSuffix = originalFilename.substring(originalFilename.lastIndexOf(".") + 1).toLowerCase();
        if(!StrUtil.equalsAny(fileSuffix,"jpg","jpeg", "png", "bmp", "gif")){
            return RestResponse.fail("文件格式不支持, 暂时只支持jpg,jpeg,png,bmp,gif");
        }
        String filename = DateUtil.today().replaceAll("-","/") + "/" +IdWorker.getIdStr() + fileSuffix;
        log.info("上传文件，新文件名：" + filename);
        boolean result = aliyunOSSUtil.putObject(systemConfiguration.bucketName,file.getBytes(),filename);
        if(result){
            String fileurl = aliyunOSSUtil.getObjectUrl(systemConfiguration.bucketName,filename);
            log.info("上传文件，文件URL：" + fileurl);
            return RestResponse.success(fileurl);
        }
        return RestResponse.fail("上传文件失败");
    }
}
